use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( examples lib script t xt ) );
__DATA__
Atria
Devel
Etheridge
INC
Joaquín
José
Karen
Kogman
Sorted
Yuval
ether
irc
jjatria
lib
nothingmuch
יובל
קוג
